from __future__ import annotations

import shutil

from typing_extensions import TYPE_CHECKING

from django.conf import settings
from django.core.management.base import (
    BaseCommand,
    CommandError
)

if TYPE_CHECKING:
    from argparse import ArgumentParser
    from pathlib import Path


class Command(BaseCommand):
    help = 'Create a custom Spire app.'

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            'app_name',
            help='Name of the application to create',
            type=str
        )

    def handle(self, *_args, **kwargs) -> None:
        root = settings.BASE_DIR

        app_name = kwargs['app_name']

        app_directory = root / 'django_spire/template/app'
        templates_directory = root / 'django_spire/template/templates'

        new_app_path = root / 'django_spire' / app_name
        new_templates_path = root / 'django_spire' / 'templates'

        if not app_directory.exists() or not templates_directory.exists():
            message = (
                'Template directories do not exist. Please ensure both "app" and '
                '"templates" exist in "django_spire/template/".'
            )

            raise CommandError(message)

        if new_app_path.exists():
            message = f'The app "{app_name}" already exists.'
            raise CommandError(message)

        shutil.copytree(app_directory, new_app_path)
        shutil.copytree(templates_directory, new_templates_path)

        self.replace_app_name(new_app_path, app_name)
        self.replace_app_name(new_templates_path, app_name)

        message = self.style.SUCCESS(f"Successfully created app '{app_name}'.")
        self.stdout.write(message)

    def replace_app_name(self, directory: Path, app_name: str) -> None:
        for path in directory.rglob('*'):
            if path.is_file():
                self.replace_content(path, app_name)
                self.rename_file(path, app_name)

    def replace_content(self, path: Path, app_name: str) -> None:
        with open(path, 'r', encoding='utf-8') as handle:
            content = handle.read()

        content = (
            content
            .replace('Placeholder', app_name.capitalize())
            .replace('Placeholders', app_name.capitalize() + 's')
            .replace('placeholder', app_name.lower())
            .replace('placeholders', app_name.lower() + 's')
        )

        with open(path, 'w') as handle:
            handle.write(content)

    def rename_file(self, path: Path, app_name: str) -> None:
        app_name_lowercase = app_name.lower()

        filename = (
            path
            .name
            .replace('placeholder', app_name_lowercase)
            .replace('placeholders', app_name_lowercase + 's')
        )

        if filename != path.name:
            new_path = path.parent / filename
            path.rename(new_path)



from __future__ import annotations

from django_spire.tests.test_cases import BaseTestCase
from django_spire.user_account import factories


class UserAccountFactoriesUnitTestCase(BaseTestCase):
    pass

    # def test_get_or_create_user_profile(self):
    #     profile = factories.get_or_create_user_profile(self.super_user)
    #     self.assertEqual(profile.user, self.super_user)



from __future__ import annotations

import argparse
import subprocess
import sys

from pathlib import Path


OMITS = (
    # Directories
    '*/system/*',
    '*/tests/*',
    '*/migrations/*',
    '*/static/*',
    '*/venv/*',
    '*/.venv/*',

    # Files
    'apps.py',
    '__init__.py',
    'manage.py',
    'automation.py',
    'run_coverage.py'
)


def main() -> None:
    # Do not ever rename this file to coverage.py as it creates an infinite
    # loop. This script must run from the root of your project.

    if sys.platform != 'win32':
        message = 'This script must run on Windows platform.'
        raise Exception(message)

    cwd = Path(__file__).parent
    virtual_env = cwd / 'venv' / 'Scripts' / 'activate'
    virtual_env_alternate = cwd / '.venv' / 'Scripts' / 'activate'

    parser = argparse.ArgumentParser(description='Run coverage with Django tests.')

    apps_help = (
        'The specific app(s) to run coverage on. '
        'The path should be relative to the project root. '
        'e.g., app/maintenance/work_order '
        'You can use a space-delimited list to include multiple apps '
        'e.g., app/maintenance/work_order/task app/maintenance/work_order/part'
    )

    parser.add_argument(
        '-a', '--apps',
        nargs='*',
        help=apps_help,
    )

    settings_help = (
        'The Django settings module. '
        'This is a Python module path that is relative to the project root. '
        'e.g., django_spire.settings'
    )

    parser.add_argument(
        '-s', '--settings',
        type=str,
        default='django_spire.settings',
        help=settings_help
    )

    venv_help = (
        'The path to the activate file for the virtual environment. '
        'The path can be absolute or relative to the project root. '
        'e.g., C:/Users/User/code/django-skeleton/venv/Scripts/activate or '
        'venv/Scripts/activate'
    )

    parser.add_argument(
        '-v', '--venv',
        type=str,
        default=virtual_env,
        help=venv_help
    )

    no_browser_help = (
        'Do not open the browser window to show the coverage results.'
    )

    parser.add_argument(
        '-nb', '--nobrowser',
        action='store_true',
        help=no_browser_help
    )

    no_erase_help = (
        'Do not erase the coverage results.'
    )

    parser.add_argument(
        '-ne', '--noerase',
        action='store_true',
        help=no_erase_help
    )

    no_html_help = 'Do not write the coverage results to HTML.'

    parser.add_argument(
        '-nh', '--nohtml',
        action='store_true',
        help=no_html_help
    )

    verbosity_help = 'Set the verbosity level of the test output.'

    parser.add_argument(
        '-vv', '--verbosity',
        type=int,
        default=1,
        help=verbosity_help
    )

    failfast_help = 'Stop the test suite after the first failed test.'

    parser.add_argument(
        '-ff', '--failfast',
        action='store_true',
        help=failfast_help
    )

    args = parser.parse_args()

    django_app_paths = ' '.join(args.apps) if args.apps else ''

    if Path(args.venv).is_file():
        activate_virtualenv_cmd = virtual_env
    elif virtual_env_alternate.is_file():
        activate_virtualenv_cmd = virtual_env_alternate
    else:
        message = (
            f'Virtual environment "{virtual_env}" or '
            f'"{virtual_env_alternate}" not found.'
        )

        raise Exception(message)

    pip_install_coverage_cmd = 'pip install coverage'

    django_settings = args.settings

    django_settings_path = cwd.joinpath(
        *django_settings.split('.')[:-1],
        f'{django_settings.split(".")[-1]}.py'
    )

    if not django_settings_path.is_file():
        message = f'Django test settings "{django_settings_path}" not found.'
        raise Exception(message)

    print('Running coverage with django tests ...\n')

    omits = ','.join(OMITS)

    coverage_run_cmd = [
        'coverage',
        'run',
        '--branch',
        '--source=.',
        f'--omit={omits}',
        'manage.py',
        'test',
        f'{django_app_paths}',
        f'--settings={django_settings}',
        '--noinput',
        f'-v {args.verbosity}'
    ]

    if args.failfast:
        coverage_run_cmd.append('--failfast')

    coverage_run_cmd = ' '.join(coverage_run_cmd)

    cmd_call = (
        f'call {activate_virtualenv_cmd}'
        f' & {pip_install_coverage_cmd}'
        f' & {coverage_run_cmd}'
    )

    html_directory = '.coverage_html_report'

    if not args.nohtml:
        coverage_html_cmd = f'coverage html --directory={html_directory}'
        cmd_call += f' & {coverage_html_cmd}'

    if not args.noerase:
        coverage_erase_cmd = 'coverage erase'
        cmd_call += f' & {coverage_erase_cmd}'

    if not args.nobrowser:
        open_browser_cmd = f'start "" "{cwd / html_directory / "index.html"}"'
        cmd_call += f' & {open_browser_cmd}'

    subprocess.run(cmd_call, check=True, shell=True)

    print('\nDone!')


if __name__ == '__main__':
    main()



from __future__ import annotations

import os
import sys

from pathlib import Path


def main() -> None:
    root = str(Path(__file__).resolve().parent.parent)
    sys.path.append(root)

    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'testing.settings')

    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        message = (
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        )

        raise ImportError(message) from exc

    execute_from_command_line(sys.argv)


if __name__ == '__main__':
    main()



from __future__ import annotations

from django_spire.tests.test_cases import BaseTestCase
from django_spire.user_account.profile.models import UserProfile
from django_spire.user_account.tests.factories import create_user


class UserProfileTestCase(BaseTestCase):
    def setUp(self):
        self.user = create_user(username='Wesley', password='goat_99')

    # def test_user_profile_receiver(self):
    #     profile = self.user.profile
    #     self.assertTrue(UserProfile.objects.filter(user=profile.user_id).exists())



from setuptools import find_packages, setup

from django_spire.constants import VERSION


with open('LICENSE.md', encoding="utf-8") as license_handle:
    license_file = license_handle.read()

with open('README.rst', encoding="utf-8") as readme_handle:
    readme_file = readme_handle.read()

setup(
    name="django-spire",
    version=VERSION,
    description="Django framework for high observability web applications",
    license=license_file,
    long_description=readme_file,
    long_description_content_type="text/x-rst",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords=[
        "spire",
        "django",
        "backend",
        "frontend",
        "javascript",
        "active server pages"
    ],
    author="Austin Sauer, Brayden Carlson, Nathan Johnson & Wesley Howery",
    author_email="info@stratusadv.com",
    url="https://github.com/stratusadv/django-spire",
    packages=find_packages(
        exclude=["docs", "example", "example.*", "tests", "tests.*"]
    ),
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.10",
    install_requires=[
        "django>=5.1.2",
        "django-glue>=0.7.8",
        "dandy>=0.4.2",
    ],
    package_data={
        "": ["README.rst", "LICENSE.md", "CHANGELOG.md", "CONTRIBUTORS.md"],
        "django_spire": [
            "**/*.html",
            "**/*.css",
            "**/*.js",
            "**/*.png",
            "**/*.jpg"
        ],
    },
    exclude_package_data={
        "": ["*.db"],
        "example": ["*"],
        "tests": ["*"],
    },
)




from __future__ import annotations

import secrets

from pathlib import Path


INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    'django_spire',
    'django_spire.authentication',
    'django_spire.authentication.mfa',
    'django_spire.breadcrumb',
    'django_spire.comment',
    'django_spire.file',
    'django_spire.form',
    'django_spire.gamification',
    'django_spire.help',
    'django_spire.history',
    'django_spire.maintenance',
    'django_spire.notification',
    'django_spire.options',
    'django_spire.pagination',
    'django_spire.permission',
    'django_spire.search',
    'django_spire.user_account',
    'django_spire.user_account.profile'
]

DEBUG = True
SECRET_KEY = secrets.token_urlsafe(50)

BASE_DIR = Path(__file__).resolve().parent.parent
SENDGRID_TEMPLATE_ID = False

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.db',
    }
}

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django_glue.middleware.GlueMiddleware'
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            str(BASE_DIR / 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django_glue.context_processors.glue'
            ],
            'builtins': [
            ],
            'debug': DEBUG,
        },
    },
]
